#!/usr/bin/perl
#
# CombineTrees
# 
# Copyright (C) 2006  Akifumi S. Tanabe
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

use strict;

my $mode = 1;
my $outtree;
my @intree;
my @treename;

# Main routine

my $buildno = '20070428';

print << "_END";
CombineTrees build $buildno
======================

Copyright (C) 2006  Akifumi S. Tanabe

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

_END

# parse options
$outtree = pop(@ARGV);
if ($outtree =~ /(?:\/|\\)/) {
	&errorMessage();
} elsif ($outtree =~ /^-+(?:help|h)/i) {
	&helpMessage();
}
foreach (@ARGV) {
	if (/^-+(?:help|h)/i) {
		&helpMessage();
	}
	elsif (/^-+(?:mode|m)=(.+)/i) {
		if ($1 =~ /^Average$/i) {
			$mode = 1;
		} elsif ($1 =~ /^Sum$/i) {
			$mode = 0;
		} else {
			&errorMessage('Unknown option.');
		}
	}
	elsif ($_ !~ /(?:\/|\\)/) {
		if (-e $_) {
			push(@intree, $_);
		} else {
			&errorMessage('Input tree file does not exist.');
		}
	} else {
		&errorMessage('Unknown option.');
	}
}

# make outtree
unless (open(OUTTREE, "> $outtree")) {
	&errorMessage();
}
print(OUTTREE "#NEXUS\n\nBegin Trees;\n");

# count trees
unless (open(INTREE, "< $intree[0]")) {
	&errorMessage();
}
my $translate;
while (<INTREE>) {
	if ($translate) {
		chomp;
		print(OUTTREE "$_\n");
		if (/;/) {
			$translate = 0;
		}
	} elsif (/tree\s+(.+?)\s*?=[^\(\)]*?\(.+\);/i) {
		push(@treename, $1);
		next;
	} elsif (/translate/i) {
		chomp;
		print(OUTTREE "$_\n");
		$translate = 1;
	}
}
close(INTREE);
close(OUTTREE);
#print("$numtree\n");

my %trees;
my %clade;
for (my $fileno = 0; $fileno < @intree; $fileno ++) {
	# read branch lengths
	unless (open(INTREE, "< $intree[$fileno]")) {
		&errorMessage('Cannot open input tree.');
	}
	my $treeno = 0;
	while (<INTREE>) {
#		print;
		if (/\((.+)\);/) {
			my $tree = $1;
			while ($tree =~ s/([^,:\(\)]+):(\d+\.?\d*)(e\D?\d+)?/$1/) {
				my $temp1 = $2;
				if ($3 =~ /e(\D)?0*(\d+)/) {
					my $temp2 = $2;
					if ($1 eq '-') {
						$temp1 *= 10 ** ((-1) * $temp2);
					} else {
						$temp1 *= 10 ** $temp2;
					}
				}
				$trees{$treename[$treeno]}{'otu'}{$1} += $temp1;
#				print("$temp1\n");
			}
			my $num = 0;
			while ($tree =~ s/\(([^:\(\)]+)\):(\d+\.?\d*)(e\D?\d+)?/<$num>$1<\/$num>/) {
				my $temp1 = $2;
				if ($3 =~ /e(\D)?0*(\d+)/) {
					my $temp2 = $2;
					if ($1 eq '-') {
						$temp1 *= 10 ** ((-1) * $temp2);
					} else {
						$temp1 *= 10 ** $temp2;
					}
				}
				$trees{$treename[$treeno]}{'clade'}{$num} += $temp1;
#				print("$temp1\n");
				$num++;
			}
#			print("$fileno:$treeno:$tree\n");
			if ($fileno == 0) {
				$tree = "<$num>" . $tree . "</$num>";
#				print("$tree\n");
				$num = 0;
				while ($tree =~ s/<(\d+)>([^<>]+)<\/\1>/\[$1\]/) {
#					print("$tree\n");
					@{$clade{$treename[$treeno]}[$num]} = split (/,/, $2);
#					print("@{$clade{$treename[$treeno]}[$num]}\n");
					$num++;
				}
			}
			$treeno ++;
		}
	}
	close(INTREE);
}

unless (open(OUTTREE, ">> $outtree")) {
	&errorMessage();
}
foreach my $treename (@treename) {
	if ($mode) {
		foreach (keys(%{$trees{$treename}{'otu'}})) {
			$trees{$treename}{'otu'}{$_} /= @intree;
		}
		foreach (keys(%{$trees{$treename}{'clade'}})) {
			$trees{$treename}{'clade'}{$_} /= @intree;
		}
	}
	# output tree
	my $outputtree = '(' . join(',', @{$clade{$treename}[@{$clade{$treename}} - 1]}) . ')';
	for (my $cladeno = @{$clade{$treename}} - 2; $cladeno >= 0; $cladeno --) {
		if ($outputtree =~ /\[$cladeno\]/) {
			my $clade = '(' . join(',', @{$clade{$treename}[$cladeno]}) . '):' . sprintf("%.12f", $trees{$treename}{'clade'}{$cladeno});
			$outputtree =~ s/\[$cladeno\]/$clade/;
		} else {
			&errorMessage();
		}
	}
	foreach (keys(%{$trees{$treename}{'otu'}})) {
		my $tempbrlens = sprintf("%.12f", $trees{$treename}{'otu'}{$_});
		$outputtree =~ s/(\(|,)$_(,|\))/$1$_:$tempbrlens$2/;
	}
	print(OUTTREE "\ttree $treename = $outputtree;\n");
}
print(OUTTREE "End;\n");
close(OUTTREE);

exit(0);

sub errorMessage {
	print("ERROR!: @_\n");
	print("If you want to read help message, run this script with \"--help\" option.\n");
	exit(1);
}

sub helpMessage {
	print << "_END";
Usage
=====

perl combinetree.pl [-m] intree1 intree2 .. intreeN outtree

Command line options
====================

-h, --help
  Display this message and exit.

-m, --mode=Average|Sum
  Specify mode. (default: Average)
_END
	exit(0);
}
